#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# 自然数 N を元にそのすべての分割を与える
# partition(3) -> (1,1,1), (2,1), (3,)
#
# $Id: partition.py,v 1.2 2003/07/15 09:38:19 etale Exp $
# =====================================================

"""library for partition"""

import sys
import itertools

# young
from mathformat import pprint_partition
import iterator

__all__     = ['partition']

# cache partitions
_cache = {}


class Partition(tuple):
    '''class for partition
    '''

    def size(self):
        # XXX
        # 必要?
        '''get the number of partition
        '''
        return sum(self)

    def __repr__(self):
        return pprint_partition(self)

    __str__ = __repr__

    def plot(self):
        '''plot the partition
        '''
        # (3,1,1)
        # -->
        # ***
        # *
        # *
        for num in self:
            print "*" * num
        else:
            print


class SeqOfPartition(iterator.Iterator):
    '''generate partitions
    '''
    def __init__(self, number):
        # validity check
        assert isinstance(number, int) and number >= 0, "number must be a non-negative integer."
        self.number = number
        self.seq    = []

    def __getitem__(self, index):
        return self.seq[index]

    def size(self):
        '''number of partitions
        '''
        # Mathematica
        # Length[]
        return len(self.seq)

    def get_partition(self):
        '''get the partition
        '''
        return self.seq

    def __iter__(self):
        return iter(self.seq)

    def set_partition(self):
        '''set partition
        '''
        try:
            self.seq = _cache[self.number]
        except KeyError, e:
            for i in range(1, self.number+1):
                try:
                    self.seq = _cache[i]
                except KeyError, e:
                    self.seq = _cache[i] = [Partition(pat) for pat in self.generate(i)]

    def plot(self):
        '''plot each partition
        '''
        for pat in self.seq:
            pat.plot()

    def generate(self, n):
        '''partition generator
        '''
        # trivial
        yield (n,)

        head, tail = n-1, 1

        while head > 0:
            if tail > 1:
                partial_result = _cache.get(tail, [])

                for candidate in partial_result[:]:
                    if _compare_max_num(head, candidate):
                        yield (head,) + candidate
            elif tail == 1:
                # tail == 1
                yield (head, tail)
            else:
                # tail == 0
                yield (head, )
            head, tail = head-1, tail+1

    def show_all_partition(self):
        '''Show all the partitions in the cache
        '''
        return show_partition(True)

    def show_partition(self, all = False):
        '''Show the partition
        If all == False,
        show each partition of the number.
        If all == True,
        show all the partitions in the cache.
        '''
        print self._show_partition(all)

    def _show_partition(self, all = False):
        buff = []
        if not all:
            buff = map(pprint_partition, self.seq)
        else:

            for i in range(1, self.number+1):

                k, v = i, _cache[i]
                buff.append(`k`)
                for item in v:
                    buff.append("\t" + pprint_partition(item))
        return "\n".join(buff)

    def __repr__(self):
        return self._show_partition()

    __str__ = __repr__


def _compare_max_num(number, part):
    # part は大きい数から順番に並んでいると仮定して、
    # number が part のどの要素よりも大きいことをチェックする。
    # 簡単にいえば、number が part の先頭の要素よりも大きいか
    # 確認すればよい.

    # number = 3
    # part = (2,1)
    # --> OK

    # number = 2
    # part = (2,1,1,1)
    # --> OK

    # number = 3
    # part = (4,2,1)
    # --> NG

    return number >= part[0]


def partition(number):
    '''Return the partition of number
    '''
    # validity check
    assert isinstance(number, int) and number > 0, "number must be a positive integer."

    par_gen = SeqOfPartition(number)
    par_gen.set_partition()
    return par_gen

def _test():
    # test

    n = partition(3)
    n.plot()

    try:
        n[:]
    except Exception, e:
        print >> sys.stderr, e
        print e.__class__

    pat = Partition((3,2,1,1))

    p = partition(4)
    print p
    for a in p:
        a.plot()

    p = partition(10)
    print "size of partition(10):", p.size()
    print "size of partition(20):", partition(20).size()

    p = partition(6)
    print "take 7"
    p.take(7)
    print "drop 8"
    p.drop(8)
    print "last 5"
    p.last(5)
    return


if __name__ == '__main__':
    _test()
